import { Client, Intents, Guild, Channel, GuildChannel, ThreadChannel, TextChannel, MessageAttachment } from 'discord.js';
import * as fs from 'fs/promises';
import * as path from "path";
const config: any = require("../config.json");
const client = new Client({ intents: [Intents.FLAGS.GUILDS] });

var allFolders: Map<string, Array<any>> = new Map<any, any>();
var collator = new Intl.Collator(undefined, {numeric: true, sensitivity: 'base'});

client.on('ready', async () => {

    // Get all directories form the import folder
    const tmpFolders: Array<string> = await fs.readdir(`./import/`);
    for ( let folder of tmpFolders ) {

        await fs.lstat(`import/${folder}`)
        .then( stat => {
            if ( stat.isDirectory() ) {
                allFolders.set(folder, []);
            }
        })
        .catch( err => {
            // pepega, ignore
            console.log(`Could not find folder/file ${folder}`);
        })
    }

    // Get all guilds
    const guilds: Array<string> = client.guilds.cache.map(guild => guild.id);

    // Go throguh all guilds and blah blah blah
    for ( let gid of guilds ) { processGuild(client, gid); }

});

// Handle guild
async function processGuild(client: Client, guid:string) {
    // Fetch the guild
    const guildData: Guild = await client.guilds.fetch(guid);

    let availableChannels: Array<TextChannel> = [];

    guildData.channels.cache.forEach( async (channel) => {
        // if threadchannel, skip
        if ( channel instanceof GuildChannel ){
            if ( allFolders.has(channel.name) ) {
                if ( channel.isText() ) {
                    console.log(`[🐹] Processing ${channel.name}`);
                    availableChannels.push(channel as TextChannel);
                }
            }
        }
    })

    for ( let channel of availableChannels ) {
        await processChannel(client, (channel as TextChannel));
    }

    console.log(`[🚗] Ending Connection...`);
    await client.destroy();
}

async function processChannel(client: Client, channel: TextChannel) {
    console.log(`Processing Channel ${channel.name}`);

    // Get all files in folder
    let fileList: Array<string> = await (getFolderFiles(`import/${channel.name}`));

    console.log(fileList);

    // Go through all of the files of the channel
    for ( let i = 0; i < fileList.length; i++ ) {
        let file = fileList[i];

        let indx: string = `${(i+1)}`.padStart(`${fileList.length}`.length, `0`);

        console.log(`[🐹][${indx}/${fileList.length}] Sending ${file} to discord...`);

        // Read file buffer
        let buff = await fs.readFile(file);
        let p = path.basename(file);
        
        let msgAttachment = new MessageAttachment(buff, p);

        await (await channel.send({
            files: [
                msgAttachment
            ]
        }));

        await fs.unlink(file)
        .catch( err => {
            console.error(`[😡] Could not delete ${file}`);
        })

        await new Promise((_r, _e) => {
            setTimeout( () => {
                _r('asd');
            }, 500);
        });
    }

}

async function getFolderFiles(folderPath: string): Promise<Array<string>> {
    let results: Array<string> = [];

    await fs.readdir(folderPath)
    .then( async files => {
        let justfiles  : Array<string> = [];
        let justFolders: Array<string> = [];

        for ( let file of files ) {
            await fs.lstat(`${folderPath}/${file}`)
            .then( async (stat) => {
    
                if ( stat.isDirectory() ) {

                    await getFolderFiles(`${folderPath}/${file}`)
                    .then( foundFiles => {
                        for ( let f of foundFiles ) {
                            justFolders.push(`${f}`);
                        }
                    })
                    .catch( err => {
                        console.error(`[😡] Failed to read file info (${file})`, err);
                    })

                } else {
                    justfiles.push(`${folderPath}/${file}`);
                }
    
            })
            .catch( err => {
                console.error(`[😡] Failed to read file info (${file})`, err);
            })
        }

        justfiles = justfiles.sort(collator.compare);

        results = justfiles.concat(justFolders);
    })
    .catch( err => {
        console.error(`[😡]`, err);
    })

    return results;
}

client.login(config.botToken);